
#ifndef __node_num__
#define __node_num__
const int NODENUM=60;
#endif

#ifndef __replica_h__
#define __replica_h__

#include <vector>



/*======================================================================================================
  REPLICA RECORD
  ======================================================================================================*/
class Replica {
  private:
        int id;
        int placement[NODENUM];
	float placement_expiration[NODENUM];
        int p_len;
	int replication_degree;
  public:
        Replica(){
                    id=-1;
                    p_len=0;
		    replication_degree=0;		    
                 };
        Replica(int ass_id)
                 {
                   id=ass_id;
                   p_len=0;
		   replication_degree=0;
                 };
        Replica(int ass_id, int ass_placement)
                 {
                   id=ass_id;
                   placement[0]=ass_placement;
                   p_len=1;
		   replication_degree=0;
                 };
		 
	//These constructors are for SRT: the manager gets the resource id and the first delegate for this replica and 
	//decides a replication degree	 
	Replica(int ass_id, int ass_placement, int ass_degree,float exp_timer)
                 {
                   id=ass_id;
                   placement[0]=ass_placement;
                   p_len=1;
		   replication_degree=ass_degree;
		   placement_expiration[0]=exp_timer;
                 };	 
	Replica(int ass_id, int ass_placement, float exp_timer)
                 {
                   id=ass_id;
                   placement[0]=ass_placement;
                   p_len=1;
		   replication_degree=0;
		   placement_expiration[0]=exp_timer;
                 };
		 
		 
        Replica(int ass_id, int* ass_placement, int ass_p_len)
                 {
                   int j=0;
                   id=ass_id;
                   for(j=0;j<ass_p_len;j++)
                      placement[j]=ass_placement[j];
                   p_len=ass_p_len;
		   replication_degree=0;
                 };
        Replica(Replica* r)
                 {
                   int j=0;
                   id=r->id;
                   for(j=0;j<r->p_len;j++)
                      {
		       placement[j]=r->placement[j];
		       placement_expiration[j]=r->placement_expiration[j];
		      }
                   p_len=r->p_len;
		   replication_degree=r->getDegree();
                 };
		 
        int getId() {return id;}
	int getPlacement(int pos){return placement[pos];}
        int* getPlacement() {return placement;}
	int getLastPlacement() {return placement[p_len-1];}
	int getDegree() {return replication_degree;}
        void addPlacement(int place){
                        placement[p_len]=place;
                        p_len++;}
        void addPlacement(Replica r){
                        int trovato=0;
                        for(int k=0;k<r.p_len;k++)
                            {
                             for(int j=0;j<p_len;j++)
                                {
                                 if(r.placement[k]==placement[j])
                                      trovato=1;
                                }
                             if(trovato==0)
                                addPlacement(r.placement[k]);
                             trovato=0;
                            }
                        } 
        
	 void addPlacementAndExpiration(Replica r){
                        int trovato=0;
                        for(int k=0;k<r.p_len;k++)
                            {
                             for(int j=0;j<p_len;j++)
                                {
                                 if(r.placement[k]==placement[j])
                                      {
				       placement_expiration[j]=r.placement_expiration[k];
				       trovato=1;
				      }
				      
                                }
                             if(trovato==0)
                                {
				 placement[p_len]=r.placement[k];
				 placement_expiration[p_len]=r.placement_expiration[k];
				 p_len++;
				}
				
                             trovato=0;
                            }
                        } 
	
	
	float getPlacementExpiration (int k){return placement_expiration[k];}
	int getPlacementLength() {return p_len;}
        int checkPlacement(int place)
                        {
                         for(int j=0;j<p_len;j++)
                            {
                             if(placement[j]==place)
                                  return 1;
                            }
                         return 0;
                        }
			
	void deletePlacement(int place)
			{
			 int k=0;
			 for(int j=0;j<p_len;j++)
                            {
                             if(placement[j]==place)
                                  {
				   for(k=j;k<p_len;k++)
				   	placement[k]=placement[k+1];
				   p_len--;
				   break;
			          }			
                            }
			}
			
	int getOwner(){
     if(p_len>0)  return placement[p_len-1];
     else return -1;
     }
	
	/*This method deletes all expired places*/
	void clean(float time)
		{
		 for(int j=0;j<p_len;)
		 	{
			 if(placement_expiration[j]<time)
			 	{
				 for(int k=j;k<p_len;k++)
				 	{
					 placement[k]=placement[k+1];
					 placement_expiration[k]=placement_expiration[k+1];
					}
				 p_len--;
				}	
			 else j++;		 			 
			}
		}
	
	
			
        };


        
class Replica_Table {
  private:
        Replica* v[NODENUM];
        int v_len;
        int pos;
	
  public:
        Replica_Table(){pos=0;
                        v_len=0;};
        void insert(Replica r){
          for(int j=0;j<v_len;j++)
              {
               if(v[j]->getId()==r.getId())
                  {
                   v[j]->addPlacementAndExpiration(r);
                   return;
                  }
              }
          Replica* r1= new Replica(&r);
          v[v_len]=r1;
          v_len++; }
        int length(){return v_len;}
        Replica get(int p) {
                    Replica r;
                    if(p<v_len) r= Replica(v[p]);
                    return r;
                    }
 
                       
        int contains(Replica r)
            {
             int j=0, found=0;
             while (found==0 && j<v_len)
                {
                 if(v[j]->getId()==r.getId()) found=1;
                 j++;
                }
             return found;
            }
        int contains(int r)
            {
             int j=0, found=0;
             while (found==0 && j<v_len)
                {
                 if(v[j]->getId()==r) found=1;
                 j++;
                }
             return found;
            }
        int contains (int r, int placement)
            {
             int j=0, found=0;
             while (found==0 && j<v_len)
                {
                 if(v[j]->getId()==r && v[j]->checkPlacement(placement))
                          found=1;
                 j++;
                }
             return found;
            }
            
        //Returns a copy of the required element
        Replica getFromId(int id)
            {
             int j=0;
             Replica r;
             while (j<v_len)
                {
                 if(v[j]->getId()==id)
                        {
                         r=Replica(v[j]);
                         return r;
                        }
                 j++;
                }
             return 0;
            }    
	    
	 void update(Replica r)
	    {
	      if(contains(r))
	      	{
		 for(int j=0;j<v_len;j++)
		 	{
		         if(v[j]->getId()==r.getId())
			 	{
				 v[j]->addPlacementAndExpiration(r);				 				 
				}
			}
		}
	      else 
	        {
		 insert(r);
		}   
	    }    
	    
	  int getResourcesOwnedBy(int node, int* resources)
	    {
	     int k=0;
	     for(int j=0;j<v_len;j++)
	     	{
                 if(v[j]->checkPlacement(node))
		 	resources[k++]=v[j]->getId();
	        }
	     return k;
	    }
	   
	  void deleteOwner(int node)
	    {
	     for(int j=0;j<v_len;j++)
	     	{
    		 if(v[j]->checkPlacement(node))
    		 	v[j]->deletePlacement(node);
    		}
	    }
	    
	  /**
	   *This method deletes all expired entries
	   */
	  void clean(float time)
	    {
	     for(int j=0;j<v_len;j++)
	     	{
     		 v[j]->clean(time);
     		}
	    }  

};
        

#endif

